<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>Webmention successful</title>
</head>
<body>
    Thank you, your webmention has been accepted
</body>
</html>
