# Generated by Django 4.1.1 on 2022-09-27 18:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("mentions", "0008_alter_webmention_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="outgoingwebmentionstatus",
            options={
                "ordering": ["-created_at"],
                "verbose_name_plural": "Outgoing Webmentions",
            },
        ),
    ]
