# Generated by Django 4.0.2 on 2022-02-03 20:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mentions', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PendingIncomingWebmention',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('source_url', models.URLField(help_text='The URL of the content that mentions your content.')),
                ('target_url', models.URLField(help_text='The URL of the page on your server that is being mentioned.')),
                ('sent_by', models.URLField(help_text='The origin of the webmention request.')),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='PendingOutgoingContent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('absolute_url', models.URLField(help_text='URL on our server where the content can be found.')),
                ('text', models.TextField(help_text='Text that may contain mentionable links. (retrieved via MentionableMixin.all_text())')),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
    ]
