H_CARD = "h-card"
H_CITE = "h-cite"
H_ENTRY = "h-entry"
H_FEED = "h-feed"

AUTHOR = "p-author"
URL = "u-url"

# Mention types
BOOKMARK = "u-bookmark-of"
LIKE = "u-like-of"
LISTEN = "u-listen-of"
REPLY = "u-in-reply-to"
REPOST = "u-repost-of"
TRANSLATION = "u-translation-of"
WATCH = "u-watch-of"
