"""
View names for use in urlpatterns and reverse lookup.
"""

"""API endpoint which receives new webmentions."""
webmention_api_incoming = "webmention_api_incoming"

"""API endpoint for retrieving existing webmentions for some target URL."""
webmention_api_get = "webmention_api_get"

"""API endpoint for retrieving existing webmentions, grouped by type, for some target URL."""
webmention_api_get_by_type = "webmention_api_get-by-type"


webmention_dashboard = "webmention-dashboard"
