"""
Extra kwargs for any `urlpatterns` entry that is expected to resolve to a
MentionableMixin instance - best left to mentions_path helpers to handle.
"""
URLPATTERNS_MODEL_NAME = "model_name"
URLPATTERNS_MODEL_FILTERS = "model_filters"
URLPATTERNS_MODEL_FILTER_MAP = "model_filter_map"
