"""API JSON keys"""
MENTIONS = "mentions"
MENTIONS_BY_TYPE = "mentions_by_type"
MESSAGE = "message"

TARGET_URL = "target_url"
SOURCE_URL = "source_url"

# Mentions
MENTION_QUOTE = "quote"
MENTION_PUBLISHED = "published"
MENTION_TYPE = "type"

MENTION_TYPE_DEFAULT = "webmention"
MENTION_TYPE_SIMPLE = "simple"

# h-card
HCARD = "hcard"
HCARD_NAME = "name"
HCARD_AVATAR = "avatar"
HCARD_HOMEPAGE = "homepage"
