# Generated by Django 4.1 on 2022-08-18 17:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mentions", "0005_rebuild_pending_models_with_constraints"),
    ]

    operations = [
        migrations.AddField(
            model_name="outgoingwebmentionstatus",
            name="is_awaiting_retry",
            field=models.BooleanField(
                default=True,
                help_text="Whether this object is eligible for another attempt at processing.",
            ),
        ),
        migrations.AddField(
            model_name="outgoingwebmentionstatus",
            name="is_retry_successful",
            field=models.BooleanField(
                default=False,
                editable=False,
                help_text="Whether this object has been processed successfully.",
            ),
        ),
        migrations.AddField(
            model_name="outgoingwebmentionstatus",
            name="last_retry_attempt",
            field=models.DateTimeField(
                blank=True,
                help_text="When we last attempted to process this object.",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="outgoingwebmentionstatus",
            name="retry_attempt_count",
            field=models.PositiveSmallIntegerField(
                default=0,
                editable=False,
                help_text="How many times we have attempted to process this object.",
            ),
        ),
        migrations.AddField(
            model_name="pendingincomingwebmention",
            name="is_awaiting_retry",
            field=models.BooleanField(
                default=True,
                help_text="Whether this object is eligible for another attempt at processing.",
            ),
        ),
        migrations.AddField(
            model_name="pendingincomingwebmention",
            name="is_retry_successful",
            field=models.BooleanField(
                default=False,
                editable=False,
                help_text="Whether this object has been processed successfully.",
            ),
        ),
        migrations.AddField(
            model_name="pendingincomingwebmention",
            name="last_retry_attempt",
            field=models.DateTimeField(
                blank=True,
                help_text="When we last attempted to process this object.",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="pendingincomingwebmention",
            name="retry_attempt_count",
            field=models.PositiveSmallIntegerField(
                default=0,
                editable=False,
                help_text="How many times we have attempted to process this object.",
            ),
        ),
    ]
