# Generated by Django 4.0.2 on 2022-06-30 18:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mentions', '0003_alter_simplemention_published_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='simplemention',
            name='post_type',
            field=models.CharField(blank=True, choices=[('bookmark', 'Bookmark'), ('like', 'Like'), ('listen', 'Listen'), ('reply', 'Reply'), ('repost', 'Repost'), ('translation', 'Translation'), ('watch', 'Watch')], help_text='Type (e.g. like, reply, etc.) of mention, if specified', max_length=64, null=True),
        ),
        migrations.AddField(
            model_name='webmention',
            name='post_type',
            field=models.CharField(blank=True, choices=[('bookmark', 'Bookmark'), ('like', 'Like'), ('listen', 'Listen'), ('reply', 'Reply'), ('repost', 'Repost'), ('translation', 'Translation'), ('watch', 'Watch')], help_text='Type (e.g. like, reply, etc.) of mention, if specified', max_length=64, null=True),
        ),
        migrations.AlterField(
            model_name='webmention',
            name='notes',
            field=models.CharField(blank=True, help_text='A description of any errors encountered when building this Webmention.', max_length=1024),
        ),
    ]
